@@ -1,6 +1,6 @@
 /* io.c - I/O operations */
  
-/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
  
 
 #include <stdio.h>
@@ -214,17 +214,25 @@ static void accept_new(void)
     vcc->active = 0;
     vcc->connecting = 0;
     vcc->fd = fd;
-#if 0
-    for (itf = itfs; itf; itf = itf->next) {
-	entry = lookup_addr(itf,&addr);
-	if (entry) {
-	    vcc->flags = entry->flags;
-	    vcc->entry = entry;
-	    Q_INSERT_HEAD(entry->vccs,vcc);
-	    return;
+    /* the following code probably belongs to arp.c ... */
+    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,pretty) <
+      0) strcpy(buffer,"<atm2text error>");
+    diag(COMPONENT,DIAG_DEBUG,"Incoming call from %s",buffer);
+    if (merge) {
+	ITF *itf;
+	for (itf = itfs; itf; itf = itf->next) {
+	    entry = lookup_addr(itf,&addr);
+	    if (entry) {
+		vcc->entry = entry;
+		Q_INSERT_HEAD(entry->vccs,vcc);
+		if (entry->state == as_valid)
+		    if (set_ip(vcc->fd,entry->ip) < 0)
+			diag(COMPONENT,DIAG_ERROR,"set_ip: %s",
+			  strerror(errno));
+		return;
+	    }
 	}
     }
-#endif
     entry = alloc_entry(1);
     entry->state = as_invalid;
     entry->addr = alloc_t(struct sockaddr_atmsvc);
@@ -232,9 +240,6 @@ static void accept_new(void)
     Q_INSERT_HEAD(unknown_incoming,entry);
     vcc->entry = entry;
     Q_INSERT_HEAD(entry->vccs,vcc);
-    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,pretty) <
-      0) strcpy(buffer,"<atm2text error>");
-    diag(COMPONENT,DIAG_DEBUG,"Incoming call from %s",buffer);
     incoming_call(vcc);
 }
 
