@@ -12,6 +12,8 @@
  */
 
 
+#include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 
 #include <netinet/in.h>
@@ -27,6 +29,9 @@
 #define MAX_ANSWER 2048
 #define MAX_NAME   1024
 
+#define MAX_LINE		2048	/* in /etc/e164_cc */
+#define E164_CC_DEFAULT_LEN	   2
+#define E164_CC_FILE		"/etc/e164_cc"
 
 #define GET16(pos) (((pos)[0] << 8) | (pos)[1])
 
@@ -101,82 +106,115 @@ static int ans(const char *text,int wanted,void *result,int res_len)
 		    return 0;
 		default:
 		    continue;
+	}
+    }
+    if (!found) return TRY_OTHER;
+    memcpy(((struct sockaddr_atmsvc *) result)->sas_addr.pub,found,
+      found_len);
+    ((struct sockaddr_atmsvc *) result)->sas_addr.pub[found_len] = 0;
+    return 0;
+}
+
+
+int ans_byname(const char *text,struct sockaddr_atmsvc *addr,int length,
+  int flags)
+{
+    if (!(flags & T2A_SVC) || length != sizeof(*addr)) return TRY_OTHER; 
+    memset(addr,0,sizeof(*addr));
+    addr->sas_family = AF_ATMSVC;
+    return ans(text,T_ATMA,addr,length);
+}
+
+
+static int encode_nsap(char *buf,const unsigned char *addr)
+{
+    static int fmt_dcc[] = { 2,12,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+      4,2,0 };
+    static int fmt_e164[] = { 2,12,1,1,1,1,1,1,1,1,16,2,0 };
+    int *fmt;
+    int pos,i,j;
+
+    switch (*addr) {
+	case ATM_AFI_DCC:
+	case ATM_AFI_ICD:
+	    fmt = fmt_dcc;
+	    break;
+	case ATM_AFI_E164:
+	    fmt = fmt_e164;
+	    break;
+	default:
+	    return TRY_OTHER;
+    }
+    pos = 2*ATM_ESA_LEN;
+    for (i = 0; fmt[i]; i++) {
+	pos -= fmt[i];
+	for (j = 0; j < fmt[i]; j++)
+	    sprintf(buf++,"%x",
+	      (addr[(pos+j) >> 1] >> 4*(1-((pos+j) & 1))) & 0xf);
+	*buf++ = '.';
+    }
+    strcpy(buf,"AESA.ATMA.INT.");
+    return 0;
+}
+
+
+int cc_len(int p0,int p1)
+{
+    static char *cc_table = NULL;
+    FILE *file;
+    char buffer[MAX_LINE];
+    char *here;
+    int cc;
+
+    if (!cc_table) {
+	if (!(cc_table = malloc(100))) {
+	    perror("malloc");
+	    return E164_CC_DEFAULT_LEN;
+	}
+	memset(cc_table,E164_CC_DEFAULT_LEN,100);
+	if (!(file = fopen(E164_CC_FILE,"r")))
+	    perror(E164_CC_FILE);
+	else {
+	    while (fgets(buffer,MAX_LINE,file)) {
+		here = strchr(buffer,'#');
+		if (here) *here = 0;
+		if (sscanf(buffer,"%d",&cc) == 1)
+		    if (cc < 10) cc_table[cc] = 1;
+		    else if (cc < 100) cc_table[cc] = 2;
+			else cc_table[cc/10] = 3;
 	    }
+	    fclose(file);
 	}
-	if (!found) return TRY_OTHER;
-	memcpy(((struct sockaddr_atmsvc *) result)->sas_addr.pub,found,found_len);
-	((struct sockaddr_atmsvc *) result)->sas_addr.pub[found_len] = 0;
-	return 0;
     }
+    if (cc_table[p0] == 1) return 1;
+    return cc_table[p0*10+p1];
+}
 
 
-    int ans_byname(const char *text,struct sockaddr_atmsvc *addr,int length,
-      int flags)
-    {
-	if (!(flags & T2A_SVC) || length != sizeof(*addr)) return TRY_OTHER; 
-	memset(addr,0,sizeof(*addr));
-	addr->sas_family = AF_ATMSVC;
-	return ans(text,T_ATMA,addr,length);
+static int encode_e164(char *buf,const char *addr)
+{
+    const char *prefix,*here;
+
+    prefix = addr+cc_len(addr[0]-48,addr[1]-48);
+    here = strchr(addr,0);
+    while (here > prefix) {
+	*buf++ = *--here;
+	*buf++ = '.';
     }
+    while (here > addr) *buf++ = *addr++;
+    strcpy(buf,".E164.ATMA.INT.");
+    return 0;
+}
 
 
-    static int encode_nsap(char *buf,const unsigned char *addr)
-    {
-	static int fmt_dcc[] = { 2,12,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
-	  4,2,0 };
-	static int fmt_e164[] = { 2,12,1,1,1,1,1,1,1,1,16,2,0 };
-	int *fmt;
-	int pos,i,j;
+int ans_byaddr(char *buffer,int length,const struct sockaddr_atmsvc *addr,
+  int flags)
+{
+    char tmp[MAX_NAME]; /* could be smaller ... */
+    int res;
 
-	switch (*addr) {
-	    case ATM_AFI_DCC:
-	    case ATM_AFI_ICD:
-		fmt = fmt_dcc;
-		break;
-	    case ATM_AFI_E164:
-		fmt = fmt_e164;
-		break;
-	    default:
-		return TRY_OTHER;
-	}
-	pos = 2*ATM_ESA_LEN;
-	for (i = 0; fmt[i]; i++) {
-	    pos -= fmt[i];
-	    for (j = 0; j < fmt[i]; j++)
-		sprintf(buf++,"%x",
-		  (addr[(pos+j) >> 1] >> 4*(1-((pos+j) & 1))) & 0xf);
-	    *buf++ = '.';
-	}
-	strcpy(buf,"AESA.ATMA.INT.");
-	return 0;
-    }
-
-
-    static int encode_e164(char *buf,const char *addr)
-    {
-	char *here;
-
-	/*
-	 * @@@ This is wrong. It should actually look up the country prefix in a
-	 * table to determine how many characters have to be copied "en block".
-	 */
-	for (here = strchr(addr,0); here > addr;) {
-	    *buf++ = *--here;
-	    *buf++ = '.';
-	}
-	strcpy(buf,"E164.ATMA.INT.");
-	return 0;
-    }
-
-
-    int ans_byaddr(char *buffer,int length,const struct sockaddr_atmsvc *addr,
-      int flags)
-    {
-	char tmp[MAX_NAME]; /* could be smaller ... */
-	int res;
-
-	if (addr->sas_addr.prv) res = encode_nsap(tmp,addr->sas_addr.prv);
-	else res = encode_e164(tmp,addr->sas_addr.pub);
-	if (res < 0) return res;
-	return ans(tmp,T_PTR,buffer,length);
-    }
+    if (addr->sas_addr.prv) res = encode_nsap(tmp,addr->sas_addr.prv);
+    else res = encode_e164(tmp,addr->sas_addr.pub);
+    if (res < 0) return res;
+    return ans(tmp,T_PTR,buffer,length);
+}
