@@ -1,6 +1,6 @@
 /* atmsigd.c - ATM signaling demon */
 
-/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
 
 
 #include <stdlib.h>
@@ -349,13 +349,24 @@ int main(int argc,char **argv)
     background = 0;
     memset(&signaling_pvc,0,sizeof(signaling_pvc));
     signaling_pvc.sap_addr.vci = 5;
+    /* 1st pass to get the -c option */
+    while ((c = getopt(argc,argv,"bc:dD:l:nNP:q:t:")) != EOF)
+	if (c == 'c') config_file = optarg;
+    if (!(yyin = fopen(config_file,"r")))
+	diag(COMPONENT,DIAG_WARN,"%s not found. - Using defaults.",config_file);
+    else if (yyparse()) {
+	    diag(COMPONENT,DIAG_FATAL,"Error in config file. - Aborting.");
+	    return 1;
+	}
+    /* process all other options but -c */
+    optind = 0;
     while ((c = getopt(argc,argv,"bc:dD:l:nNP:q:t:")) != EOF)
 	switch (c) {
 	    case 'b':
 		background = 1;
 		break;
 	    case 'c':
-		config_file = optarg;
+		/* already handled */
 		break;
 	    case 'd':
 		set_verbosity(NULL,DIAG_DEBUG);
@@ -405,12 +416,6 @@ int main(int argc,char **argv)
 #endif
 #endif
       "/AAL5, version " VERSION);
-    if (!(yyin = fopen(config_file,"r")))
-	diag(COMPONENT,DIAG_WARN,"%s not found. - Using defaults.",config_file);
-    else if (yyparse()) {
-	    diag(COMPONENT,DIAG_FATAL,"Error in config file. - Aborting.");
-	    return 1;
-	}
     if (dump_dir)
 	if (chdir(dump_dir) < 0)
 	    diag(COMPONENT,DIAG_ERROR,"chdir %s: %s",dump_dir,strerror(errno));
