@@ -1,6 +1,6 @@
 /* q2931.c - Processing of incoming Q.2931 messages */
  
-/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
  
 
 #include <stdarg.h>
@@ -18,6 +18,7 @@
 #include "sap.h"
 #include "io.h"
 #include "timeout.h"
+#include "trace.h"
 
 #define __KERNEL__ /* since that's what we effectively are */
 #include <linux/errno.h>
@@ -229,6 +230,7 @@ static void send_drop_party_ack(unsigned long call_ref,short ep_ref,
 
 static void q2931_call(SOCKET *sock,unsigned char mid)
 {
+    char buffer[MAX_ATM_ADDR_LEN+1];
     int error;
 
     switch (mid) {
@@ -312,8 +314,10 @@ static void q2931_call(SOCKET *sock,unsigned char mid)
 	    send_kernel(sock->id,0,as_okay,0,&sock->pvc,NULL,sock->local,
 	      &sock->qos);
 	    new_state(sock,ss_connected);
-	    diag(COMPONENT,DIAG_INFO,"Active open succeeded (CR 0x%06X)",
-	      sock->call_ref);
+	    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *)
+	      sock->local,0) < 0) strcpy(buffer,"<invalid>");
+	    diag(COMPONENT,DIAG_INFO,"Active open succeeded (CR 0x%06X, "
+	      "ID 0x%08x, addr %s)",sock->call_ref,sock->id,buffer);
 	    return;
 	case QMSG_CONN_ACK: /* ACCEPTING, WAIT_REL, REL_REQ */
 	    diag(COMPONENT,DIAG_DEBUG,"CA vpi.vci=%d.%d",
@@ -326,8 +330,10 @@ static void q2931_call(SOCKET *sock,unsigned char mid)
 	    STOP_TIMER(sock);
 	    send_kernel(sock->id,0,as_okay,0,NULL,NULL,sock->local,NULL);
 	    new_state(sock,ss_connected);
-	    diag(COMPONENT,DIAG_INFO,"Passive open succeeded (CR 0x%06X)",
-	      sock->call_ref);
+	    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1, (struct sockaddr *)
+	      sock->local,0) < 0) strcpy(buffer,"<invalid>");
+	    diag(COMPONENT,DIAG_INFO,"Passive open succeeded (CR 0x%06X, "
+	      "ID 0x%08x, addr %s)",sock->call_ref,sock->id,buffer);
 	    return;
 	case QMSG_RELEASE: /* all states */
 	    {
@@ -462,6 +468,7 @@ void saal_failure(void)
 {
     SOCKET *curr,*next;
 
+    trace_msg("SAAL went down");
     for (curr = sockets; curr; curr = next) {
 	next = curr->next;
 	if (curr->q2931_state != qs_null)
@@ -476,6 +483,7 @@ void saal_okay(void)
 {
     SOCKET *curr;
 
+    trace_msg("SAAL came up");
     if (!t309) return;
     stop_timer(t309);
     t309 = NULL;
